@@ -30,6 +30,39 @@ using namespace icu;
 
 namespace Transfuse {
 
+inline void append_xml(UnicodeString& str, const UnicodeString& xc, bool nls = false) {
+	for (int32_t i = 0; i < xc.length(); ++i) {
+		auto c = xc[i];
+		if (c == '&') {
+			str.append(u"&amp;", 0, -1);
+		}
+		else if (c == '"') {
+			str.append(u"&quot;", 0, -1);
+		}
+		else if (c == '\'') {
+			str.append(u"&apos;", 0, -1);
+		}
+		else if (c == '<') {
+			str.append(u"&lt;", 0, -1);
+		}
+		else if (c == '>') {
+			str.append(u"&gt;", 0, -1);
+		}
+		else if (c == '\t' && nls) {
+			str.append(u"&#9;", 0, -1);
+		}
+		else if (c == '\n' && nls) {
+			str.append(u"&#10;", 0, -1);
+		}
+		else if (c == '\r' && nls) {
+			str.append(u"&#13;", 0, -1);
+		}
+		else {
+			str += c;
+		}
+	}
+}
+
 std::unique_ptr<DOM> extract_html(State& state, std::unique_ptr<icu::UnicodeString> data) {
 	if (!data) {
 		auto raw_data = file_load("original");
@@ -124,6 +157,23 @@ std::unique_ptr<DOM> extract_html(State& state, std::unique_ptr<icu::UnicodeStri
 		rx_subp_close.reset(*data);
 		tmp = rx_subp_close.replaceAll("<$1 tf-added-after=\"1\"$2$3$4 $5", status);
 		std::swap(tmp, *data);
+
+		tmp.remove();
+		RegexMatcher rx_cdata(R"X(<!\[CDATA\[(.*?)\]\]>)X", UREGEX_DOTALL, status);
+		rx_cdata.reset(*data);
+		int32_t last = 0;
+		while (rx_cdata.find()) {
+			auto b = rx_cdata.start(0, status);
+			tmp += data->tempSubStringBetween(last, b);
+
+			b = rx_cdata.start(1, status);
+			auto e = rx_cdata.end(1, status);
+			append_xml(tmp, data->tempSubStringBetween(b, e));
+
+			last = rx_cdata.end(0, status);
+		}
+		tmp += data->tempSubStringBetween(last);
+		std::swap(tmp, *data);
 	}
 
 	auto xml = htmlReadMemory(reinterpret_cast<const char*>(data->getTerminatedBuffer()), SI(SZ(data->length()) * sizeof(UChar)), "transfuse.html", utf16_native, HTML_PARSE_RECOVER | HTML_PARSE_NOWARNING | HTML_PARSE_NOERROR | HTML_PARSE_NONET);
@@ -186,10 +236,10 @@ std::string inject_html(DOM& dom) {
 	while (b != std::string::npos) {
 		auto e = content.find(TFU_CLOSE, b);
 		std::string_view hash{&content[b + 3], e - b - 3};
-		auto body = dom.state.style("U", hash);
+		auto [topen, tclose, _] = dom.state.style("U", hash);
 		content.erase(content.begin() + PD(b), content.begin() + PD(e) + 3);
-		content.insert(content.begin() + PD(b), body.second.begin(), body.second.end());
-		content.insert(content.begin() + PD(b), body.first.begin(), body.first.end());
+		content.insert(content.begin() + PD(b), tclose.begin(), tclose.end());
+		content.insert(content.begin() + PD(b), topen.begin(), topen.end());
 		b = content.find(TFU_OPEN);
 	}
 
