@@ -83,8 +83,8 @@ void pptx_merge_at(State& state, xmlDocPtr xml) {
 
 			auto bp = node->parent;
 			xmlBufferEmpty(buf);
-			auto sz = xmlNodeDump(buf, bp->doc, bp, 0, 0);
-			tag.assign(buf->content, buf->content + sz);
+			xmlNodeDump(buf, bp->doc, bp, 0, 0);
+			tag.assign(buf->content, buf->content + buf->use);
 
 			xmlChar_view type{ XC("text") };
 			auto apos = tag.find(XC("a:hlinkClick"));
@@ -229,7 +229,7 @@ std::unique_ptr<DOM> extract_pptx(State& state) {
 	xmlSaveFileTo(obuf, xml, "UTF-8");
 	data.assign(buf->content, buf->content + buf->use);
 	xmlBufferFree(buf);
-	cleanup_styles(data);
+	cleanup_styles(state, data);
 
 	auto b = data.rfind("</tf-text><tf-text>");
 	while (b != std::string::npos) {
