@@ -32,13 +32,13 @@ static void escape_body(xmlString& s, std::string_view xc) {
 	for (size_t i = 0; i < xc.size(); ++i) {
 		if (xc[i] == '\xee' && xc[i + 1] == '\x80' && xc[i + 2] >= '\x91' && xc[i + 2] <= '\x93') {
 			if (xc[i + 2] == '\x91') {
-				s += "\n<STYLE:";
+				s += "<STYLE:";
 			}
 			else if (xc[i + 2] == '\x92') {
-				s += ">\n";
+				s += ">";
 			}
 			else if (xc[i + 2] == '\x93') {
-				s += "\n</STYLE>\n";
+				s += "</STYLE>";
 			}
 			i += 2;
 			continue;
@@ -52,43 +52,50 @@ static void escape_body(xmlString& s, xmlChar_view xc) {
 
 // Turns protected tags to inline tags on the surrounding tokens
 void VISLStream::protect_to_styles(xmlString& styled, State& state) {
+	if (state.settings->opt_verbose) {
+		std::cerr << "Protected to inline (VISL)" << std::endl;
+	}
+
 	UText tmp_ut = UTEXT_INITIALIZER;
 	UErrorCode status = U_ZERO_ERROR;
 
 	// Merge protected regions if they only have whitespace between them
-	auto rx_prots = std::make_unique<RegexMatcher>(R"X(\uE021([\s\r\n\p{Z}]*)\uE020)X", 0, status);
+	RegexMatcher rx_pmerge(R"X(\uE021([\s\r\n\p{Z}]*)\uE020)X", 0, status);
 
 	utext_openUTF8(tmp_ut, styled);
-	rx_prots->reset(&tmp_ut);
+	rx_pmerge.reset(&tmp_ut);
 
 	xmlString ns;
 	ns.reserve(styled.size());
 
 	int32_t last = 0;
-	while (rx_prots->find()) {
-		auto b = rx_prots->start(status);
+	while (rx_pmerge.find()) {
+		auto b = rx_pmerge.start(status);
 		ns.append(styled.begin() + last, styled.begin() + b);
-		auto b1 = rx_prots->start(1, status);
-		auto e1 = rx_prots->end(1, status);
+		auto b1 = rx_pmerge.start(1, status);
+		auto e1 = rx_pmerge.end(1, status);
 		ns.append(styled.begin() + b1, styled.begin() + e1);
-		last = rx_prots->end(status);
+		last = rx_pmerge.end(status);
 	}
 	ns.append(styled.begin() + last, styled.end());
 
 	styled.swap(ns);
 
 	// Find all protected regions and convert them to styles on the surrounding tokens
-	rx_prots = std::make_unique<RegexMatcher>(R"X(\uE020(.*?)\uE021)X", UREGEX_DOTALL, status);
+	RegexMatcher rx_prots(R"X(\uE020(.*?)\uE021)X", UREGEX_DOTALL, status);
 	RegexMatcher rx_block_start(R"X(>[\s\p{Zs}]*$)X", 0, status);
 	RegexMatcher rx_block_end(R"X(^[\s\p{Zs}]*<)X", 0, status);
 
+	RegexMatcher rx_tag_start(R"X(<([-:_\p{L}\p{N}\p{M}]+))X", 0, status);
+
 	RegexMatcher rx_pfx_style(R"X(\ue013[\s\p{Zs}]*$)X", 0, status);
-	RegexMatcher rx_pfx_token(R"X([^>\s\p{Z}\ue012]+[\s\p{Zs}]*$)X", 0, status);
+	RegexMatcher rx_pfx_token(R"X([^<>\s\p{Z}\ue011-\ue013]+[\s\p{Zs}]*$)X", 0, status);
+	RegexMatcher rx_sfx_token(R"X(^[\s\p{Zs}]*[^<>\s\p{Z}\ue011-\ue013]+)X", 0, status);
 
 	RegexMatcher rx_ifx_start(R"X((\ue011[^\ue012]+\ue012)[\s\p{Zs}]*$)X", 0, status);
 
 	utext_openUTF8(tmp_ut, styled);
-	rx_prots->reset(&tmp_ut);
+	rx_prots.reset(&tmp_ut);
 
 	ns.resize(0);
 	ns.reserve(styled.size());
@@ -96,22 +103,28 @@ void VISLStream::protect_to_styles(xmlString& styled, State& state) {
 
 	UText tmp_pfx = UTEXT_INITIALIZER;
 	UText tmp_sfx = UTEXT_INITIALIZER;
+	UText tmp_p = UTEXT_INITIALIZER;
+	int64_t ni = 0;
 	for (size_t i = 0; i < 100; ++i) {
 		last = 0;
-		while (rx_prots->find(last, status)) {
-			auto b = rx_prots->start(status);
+		while (rx_prots.find(last, status)) {
+			auto b = rx_prots.start(status);
 			ns.append(styled.begin() + last, styled.begin() + b);
 
-			auto b1 = rx_prots->start(1, status);
-			auto e1 = rx_prots->end(1, status);
+			auto b1 = rx_prots.start(1, status);
+			auto e1 = rx_prots.end(1, status);
 			tmp_lxs[0].assign(styled.begin() + b1, styled.begin() + e1);
-			last = rx_prots->end(status);
+			utext_openUTF8(tmp_p, tmp_lxs[0]);
+			last = rx_prots.end(status);
 
+			auto sfx = xmlChar_view(styled).substr(SZ(last));
 			utext_openUTF8(tmp_pfx, ns);
-			utext_openUTF8(tmp_sfx, xmlChar_view(styled).substr(SZ(last)));
+			utext_openUTF8(tmp_sfx, sfx);
 
+			utext_setNativeIndex(&tmp_pfx, std::max(SI32(ns.size()) - 100, 0));
+			ni = utext_getNativeIndex(&tmp_pfx);
 			rx_block_start.reset(&tmp_pfx);
-			if (rx_block_start.find()) {
+			if (rx_block_start.find(ni, status)) {
 				// If we are at the beginning of a block tag, just leave the protected inline as-is
 				ns += tmp_lxs[0];
 				continue;
@@ -124,8 +137,34 @@ void VISLStream::protect_to_styles(xmlString& styled, State& state) {
 				continue;
 			}
 
+			auto hash = state.style(XC("P"), tmp_lxs[0], XC(""), "P");
+			ns += TFI_OPEN_B "P:";
+			ns += hash;
+			ns += TFI_OPEN_E;
+
+			bool had_tags = false;
+			rx_tag_start.reset(&tmp_p);
+			while (rx_tag_start.find()) {
+				auto tb = rx_tag_start.start(1, status);
+				auto te = rx_tag_start.end(1, status);
+				ns += TFP_STREAM_B;
+				ns.append(tmp_lxs[0].begin() + tb, tmp_lxs[0].begin() + te);
+				ns += TFP_STREAM_E;
+				had_tags = true;
+			}
+			if (!had_tags) {
+				ns += TFP_STREAM_B;
+				ns += "xml-special";
+				ns += TFP_STREAM_E;
+			}
+
+			ns += TFI_CLOSE;
+
+			/*
+			utext_setNativeIndex(&tmp_pfx, std::max(SI32(ns.size()) - 100, 0));
+			ni = utext_getNativeIndex(&tmp_pfx);
 			rx_ifx_start.reset(&tmp_pfx);
-			if (rx_ifx_start.find()) {
+			if (rx_ifx_start.find(ni, status)) {
 				// We're inside at the start of an existing style, so wrap whole inside
 				auto hash = state.style(XC("P"), tmp_lxs[0], XC(""));
 				auto last_s = rx_ifx_start.end(1, status);
@@ -142,8 +181,10 @@ void VISLStream::protect_to_styles(xmlString& styled, State& state) {
 				continue;
 			}
 
+			utext_setNativeIndex(&tmp_pfx, std::max(SI32(ns.size()) - 100, 0));
+			ni = utext_getNativeIndex(&tmp_pfx);
 			rx_pfx_style.reset(&tmp_pfx);
-			if (rx_pfx_style.find()) {
+			if (rx_pfx_style.find(ni, status)) {
 				// Create a new style around the immediately preceding style
 				auto hash = state.style(XC("P"), XC(""), tmp_lxs[0]);
 				auto last_s = ns.rfind(XC(TFI_OPEN_B));
@@ -157,11 +198,32 @@ void VISLStream::protect_to_styles(xmlString& styled, State& state) {
 				continue;
 			}
 
+			utext_setNativeIndex(&tmp_pfx, std::max(SI32(ns.size()) - 100, 0));
+			ni = utext_getNativeIndex(&tmp_pfx);
 			rx_pfx_token.reset(&tmp_pfx);
-			if (rx_pfx_token.find()) {
+			if (rx_pfx_token.find(ni, status)) {
 				// Create a new style around the immediately preceding token
 				auto hash = state.style(XC("P"), XC(""), tmp_lxs[0]);
 				auto last_s = rx_pfx_token.start(status);
+
+				if (last_s <= std::max(SI32(ns.size()) - 100, 0)) {
+					// If we are in the middle of a very long token (e.g. URL), encompass the whole token
+					UChar32 cp = 0;
+					utext_setNativeIndex(&tmp_pfx, last_s);
+					while ((cp = UTEXT_PREVIOUS32(&tmp_pfx)) != U_SENTINEL && (cp != '>' && cp != TFI_CLOSE_UC && !u_isWhitespace(cp))) {
+						// UTEXT_PREVIOUS32 already changed tmp_pfx
+					}
+					UTEXT_NEXT32(&tmp_pfx);
+					last_s = SI(utext_getNativeIndex(&tmp_pfx));
+				}
+				else {
+					// Move to next entity if there is one, to avoid cutting entities in half
+					auto amp = ns.find('&', last_s);
+					if (amp != xmlString::npos) {
+						last_s = SI(amp);
+					}
+				}
+
 				tmp_lxs[1] = ns.substr(SZ(last_s));
 				ns.resize(SZ(last_s));
 				ns += TFI_OPEN_B "P:";
@@ -171,6 +233,26 @@ void VISLStream::protect_to_styles(xmlString& styled, State& state) {
 				ns += TFI_CLOSE;
 				continue;
 			}
+
+			rx_sfx_token.reset(&tmp_sfx);
+			if (rx_sfx_token.find()) {
+				// Create a new style around the immediately succeeding token
+				auto hash = state.style(XC("P"), tmp_lxs[0], XC(""));
+				auto e = rx_sfx_token.end(status);
+				tmp_lxs[1] = sfx.substr(0, e);
+				ns += TFI_OPEN_B "P:";
+				ns += hash;
+				ns += TFI_OPEN_E;
+				ns += tmp_lxs[1];
+				ns += TFI_CLOSE;
+				last += e;
+				continue;
+			}
+
+			if (state.settings->opt_verbose) {
+				std::cerr << "Could not attach protected tag: " << XV2SV(tmp_lxs[0]) << std::endl;
+			}
+			//*/
 		}
 
 		if (last == 0) {
@@ -180,7 +262,7 @@ void VISLStream::protect_to_styles(xmlString& styled, State& state) {
 		ns.append(styled.begin() + last, styled.end());
 		styled.swap(ns);
 		utext_openUTF8(tmp_ut, styled);
-		rx_prots->reset(&tmp_ut);
+		rx_prots.reset(&tmp_ut);
 		ns.resize(0);
 		ns.reserve(styled.size());
 	}
@@ -230,27 +312,38 @@ std::istream& VISLStream::get_block(std::istream& in, std::string& str, std::str
 	while (std::getline(in, buffer)) {
 		auto bb = buffer.find("<s id=\"");
 		auto eb = buffer.find("\">");
-		auto bs = buffer.find("<STYLE:");
 		if (bb == 0 && eb != std::string::npos) {
 			block_id.assign(buffer.begin() + 7, buffer.begin() + PD(eb));
 			continue;
 		}
-		else if (bs == 0) {
-			trim(buffer);
-			str += TFI_OPEN_B;
-			str.append(buffer.begin() + 7, buffer.end() - 1);
-			str += ";";
-			str += TFI_OPEN_E;
-			continue;
-		}
-		else if (buffer.compare("</STYLE>") == 0) {
-			str += TFI_CLOSE;
-			continue;
-		}
 		else if (buffer.compare("</s>") == 0) {
 			break;
 		}
-		str += buffer;
+
+		std::string_view buf{ buffer };
+		while (!buf.empty()) {
+			auto bs = buf.find("<STYLE:");
+			auto es = buf.find("</STYLE>");
+			if (es != std::string_view::npos && (bs == std::string_view::npos || es < bs)) {
+				str.append(buf.begin(), buf.begin() + es);
+				buf.remove_prefix(es + 8);
+				str += TFI_CLOSE;
+				continue;
+			}
+			if (bs != std::string_view::npos && (es == std::string_view::npos || bs < es)) {
+				str.append(buf.begin(), buf.begin() + bs);
+				str += TFI_OPEN_B;
+				buf.remove_prefix(bs + 7);
+				auto c = buf.find('>');
+				str.append(buf.begin(), buf.begin() + c);
+				buf.remove_prefix(c + 1);
+				str += ";";
+				str += TFI_OPEN_E;
+				continue;
+			}
+			break;
+		}
+		str += buf;
 	}
 	return in;
 }
