@@ -20,25 +20,21 @@
 #define e5bd51be_STREAM_HPP_
 
 #include "filesystem.hpp"
-#include "string_view.hpp"
 #include "xml.hpp"
 #include "shared.hpp"
 #include "state.hpp"
 #include <unicode/utext.h>
 #include <vector>
 #include <string>
+#include <string_view>
 #include <fstream>
 
 namespace Transfuse {
 
-namespace Streams {
-	const std::string_view detect{ "detect" };
-	const std::string_view apertium{ "apertium" };
-	const std::string_view visl{ "visl" };
-}
-using Stream = std::string_view;
-
 struct StreamBase {
+	Settings* settings = nullptr;
+
+	StreamBase(Settings* settings) : settings(settings) {}
 	virtual ~StreamBase() = default;
 
 	// Output functions
@@ -46,6 +42,7 @@ struct StreamBase {
 	virtual void stream_header(xmlString&, fs::path) = 0;
 	virtual void block_open(xmlString&, xmlChar_view) = 0;
 	virtual void block_body(xmlString&, xmlChar_view) = 0;
+	virtual void block_term_header(xmlString&) = 0;
 	virtual void block_close(xmlString&, xmlChar_view) = 0;
 
 	// Input functions
@@ -53,17 +50,20 @@ struct StreamBase {
 	virtual std::istream& get_block(std::istream&, std::string&, std::string&) = 0;
 };
 
-struct ApertiumStream final : StreamBase {
+struct ApertiumStream : StreamBase {
+	ApertiumStream(Settings* settings) : StreamBase(settings) {}
+
 	// Output functions
-	void protect_to_styles(xmlString&, State&) final;
-	void stream_header(xmlString&, fs::path) final;
-	void block_open(xmlString&, xmlChar_view) final;
-	void block_body(xmlString&, xmlChar_view) final;
-	void block_close(xmlString&, xmlChar_view) final;
+	void protect_to_styles(xmlString&, State&);
+	void stream_header(xmlString&, fs::path);
+	void block_open(xmlString&, xmlChar_view);
+	void block_body(xmlString&, xmlChar_view);
+	void block_term_header(xmlString&);
+	void block_close(xmlString&, xmlChar_view);
 
 	// Input functions
-	fs::path get_tmpdir(std::string&) final;
-	std::istream& get_block(std::istream&, std::string&, std::string&) final;
+	fs::path get_tmpdir(std::string&);
+	std::istream& get_block(std::istream&, std::string&, std::string&);
 
 private:
 	std::vector<std::string> wbs;
@@ -71,22 +71,32 @@ private:
 	std::string unesc;
 };
 
-struct VISLStream final : StreamBase {
+struct VISLStream : StreamBase {
+	VISLStream(Settings* settings) : StreamBase(settings) {}
+
 	// Output functions
-	void protect_to_styles(xmlString&, State&) final;
-	void stream_header(xmlString&, fs::path) final;
-	void block_open(xmlString&, xmlChar_view) final;
-	void block_body(xmlString&, xmlChar_view) final;
-	void block_close(xmlString&, xmlChar_view) final;
+	void protect_to_styles(xmlString&, State&);
+	void stream_header(xmlString&, fs::path);
+	void block_open(xmlString&, xmlChar_view);
+	void block_body(xmlString&, xmlChar_view);
+	void block_term_header(xmlString&);
+	void block_close(xmlString&, xmlChar_view);
 
 	// Input functions
-	fs::path get_tmpdir(std::string&) final;
-	std::istream& get_block(std::istream&, std::string&, std::string&) final;
+	fs::path get_tmpdir(std::string&);
+	std::istream& get_block(std::istream&, std::string&, std::string&);
 
-private:
+protected:
 	std::string buffer;
 };
 
+struct CGStream : VISLStream {
+	CGStream(Settings* settings) : VISLStream(settings) {}
+
+	// Input functions
+	std::istream& get_block(std::istream&, std::string&, std::string&);
+};
+
 inline void utext_openUTF8(UText& ut, xmlChar_view xc) {
 	UErrorCode status = U_ZERO_ERROR;
 	utext_openUTF8(&ut, reinterpret_cast<const char*>(xc.data()), SI64(xc.size()), &status);
