@@ -87,9 +87,12 @@ int main(int argc, char* argv[]) {
 		O('i',   "input", ARG_REQ, "input file, if not passed as arg; default and - is stdin"),
 		O('o',  "output", ARG_REQ, "output file, if not passed as arg; default and - is stdout"),
 		O('H', "mark-headers", ARG_NO, "output U+2761 after headers, such as HTML tags h1-h6 and attribute 'title'"),
+		O('v', "verbose",  ARG_NO, "more information about steps and progress"),
+		O(0,     "debug",  ARG_NO, "write debug files in state folder"),
 		O('V', "version",  ARG_NO, "output version information"),
 		O(0,   "apertium-n", ARG_NO, "apertium -n mode to prevent appending .[] to blocks"),
 		O(0,   "inject-raw", ARG_NO, "inserts as verbatim as possible, as XML fragments"),
+		O(0,   "no-extend",  ARG_NO, "don't extend inline tags to surrounding alphanumerics"),
 		spacer(),
 		text("Hook programs are called with a filename as first argument. After the hook exits, Transfuse reads the same filename and uses the contents as-is."),
 		spacer(),
@@ -103,6 +106,8 @@ int main(int argc, char* argv[]) {
 		O(0,   "tags-prot-inline", ARG_REQ, "inline protected tags; will be attached to a token as a P marker; e.g. HTML br"),
 		O(0,   "tags-raw", ARG_REQ, "CDATA tags that shouldn't be XML-encoded in the result; e.g. HTML script, style"),
 		O(0,   "tags-inline", ARG_REQ, "inline tags; formatting that shouldn't cause sentence breaks; e.g. HTML a, i, b"),
+		O(0,   "tags-semantic", ARG_REQ, "inline tags that should be emitted even if they're a full block"),
+		O(0,   "tags-unique", ARG_REQ, "tags that must not be merged even if adjacent; e.g. TEI seg"),
 		O(0,   "tags-parents-allow", ARG_REQ, "if set, only extract children of these tags; e.g. ODT text:h, text:p"),
 		//O(0,   "tags-parents-direct", ARG_REQ, "not sure, maybe remnant from TTX"),
 		O(0,   "tag-attrs", ARG_REQ, "attributes that should be extracted as separate segments; e.g. HTML alt, label"),
@@ -196,6 +201,9 @@ int main(int argc, char* argv[]) {
 		case 'o':
 			settings.out = write_or_stdout(o->value.data(), settings._out);
 			break;
+		case 'v':
+			settings.opt_verbose = true;
+			break;
 		}
 		if (o->longopt == "apertium-n") {
 			settings.opt_apertium_n = true;
@@ -206,6 +214,12 @@ int main(int argc, char* argv[]) {
 		else if (o->longopt == "hook-inject") {
 			settings.hook_inject = o->value;
 		}
+		else if (o->longopt == "no-extend") {
+			settings.opt_no_extend = true;
+		}
+		else if (o->longopt == "debug") {
+			settings.opt_debug = true;
+		}
 	}
 
 	for (auto mt : maybe_tags) {
@@ -257,6 +271,9 @@ int main(int argc, char* argv[]) {
 	auto curdir = fs::current_path();
 
 	if (settings.mode == "clean") {
+		if (settings.opt_verbose) {
+			std::cerr << "Mode: clean" << std::endl;
+		}
 		// Extracts and immediately injects again - useful for cleaning documents for other CAT tools, such as OmegaT
 		extract(settings);
 		settings.in = read_or_stdin("extracted", settings._in);
@@ -268,6 +285,9 @@ int main(int argc, char* argv[]) {
 		settings.tmpdir = rv.first;
 	}
 	else if (settings.mode == "extract") {
+		if (settings.opt_verbose) {
+			std::cerr << "Mode: extract" << std::endl;
+		}
 		extract(settings);
 		std::ifstream data("extracted", std::ios::binary);
 		data.exceptions(std::ios::badbit | std::ios::failbit);
@@ -275,6 +295,9 @@ int main(int argc, char* argv[]) {
 		settings.out->flush();
 	}
 	else if (settings.mode == "inject") {
+		if (settings.opt_verbose) {
+			std::cerr << "Mode: inject" << std::endl;
+		}
 		settings.in = read_or_stdin(settings.infile, settings._in);
 		auto rv = inject(settings);
 		std::ifstream data(rv.second, std::ios::binary);
@@ -286,6 +309,9 @@ int main(int argc, char* argv[]) {
 
 	// If neither --dir nor --keep, wipe the temporary folder
 	if (!settings.opt_keep && (settings.mode == "clean" || settings.mode == "inject")) {
+		if (settings.opt_verbose) {
+			std::cerr << "Removing folder " << settings.tmpdir << std::endl;
+		}
 		fs::current_path(curdir);
 		fs::remove_all(settings.tmpdir);
 	}
